'use strict';

/**
 * Copyright 2015 Urban Airship Inc. and Contributors. Subject to the LICENSE
 * file at the top-level directory of this distribution and at
 * https://github.com/urbanairship/frock/blob/master/LICENSE
 */
var net = require('net');

var bole = require('bole');
var enableDestroy = require('server-destroy');

var _require = require('./utils'),
    handleServerError = _require.handleServerError,
    createConnectionFilter = _require.createConnectionFilter;

var log = bole('frock/core-socketserver');

module.exports = createSocketServer;

function createSocketServer(frock, config, globalConfig, ready) {
  var deter = createConnectionFilter(frock, config, globalConfig, onSocketWhitelistFail);

  var handler = void 0;
  var server = void 0;

  try {
    frock.handlers.register(config.handler);
  } catch (e) {
    log.error('error registering socket handler ' + config.handler, e);

    return ready(e);
  }

  var logId = config.handler + ':' + config.port + '>';
  var handlerFn = frock.handlers.get(config.handler);

  if (!config.options) {
    config.options = {};
  }

  handler = handlerFn(frock, bole(logId), config.options, config.db ? frock.dbs.register(config.db) : null);

  server = net.createServer(deter(handler));

  log.debug('added socket [' + config.handler + ']');

  server.on('error', handleServerError(log, config));
  server.listen(config.port);
  enableDestroy(server);

  ready(null, { server: server, handlers: [handler], middlewares: [], port: config.port });

  log.info('started server ' + config.port);
}

// testing exports
createSocketServer._onSocketWhitelistFail = onSocketWhitelistFail;

function onSocketWhitelistFail(client) {
  client.end();
  log.info('access socket from non-whitelisted, or from blacklisted address. ' + 'see https://github.com/urbanairship/frock/blob/master/docs/frockfile.md#connection-object-optional ' + 'for further details.');
}