'use strict';

/**
 * Copyright 2015 Urban Airship Inc. and Contributors. Subject to the LICENSE
 * file at the top-level directory of this distribution and at
 * https://github.com/urbanairship/frock/blob/master/LICENSE
 */
var bole = require('bole');

var _require2 = require('resolve'),
    resolve = _require2.sync;

var log = bole('frock/cores');

module.exports = createCores;

function createCores(frock) {
  var _require = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : require;

  var localPkg = {};

  try {
    localPkg = _require(resolve('./package.json', { basedir: frock.pwd }));
  } catch (e) {
    log.debug('could not locate a project-local package.json');
  }

  var cores = localPkg.frock && localPkg.frock.cores || [];

  return cores.map(createCore);

  function createCore(config) {
    var logId = 'core::' + config.handler + '>';

    var handler = void 0;
    var handlerFn = void 0;

    try {
      frock.handlers.register(config.handler);
    } catch (e) {
      log.error('error registering core ' + config.handler, e);

      return;
    }

    handlerFn = frock.handlers.get(config.handler);

    if (!config.options) {
      config.options = {};
    }

    handler = handlerFn(frock, bole(logId), config.options, config.db ? frock.dbs.register(config.db) : null);

    return handler;
  }
}